#define SAMPLES 64.

varying mediump vec2 var_texcoord0;

uniform lowp vec4 enable_effects1;
uniform lowp vec4 enable_effects2;

uniform lowp vec4 effects1;
uniform lowp vec4 effects2;
uniform lowp vec4 effects3;
uniform lowp vec4 effects4;
uniform lowp vec4 effects5;
uniform lowp vec4 effects6;

uniform lowp sampler2D tex0;
uniform lowp vec4 tint;

#define BLOOM enable_effects1.x
#define CHROMATIC enable_effects1.y
#define VIGNETTE enable_effects1.z
#define NOISE enable_effects1.w

#define VHS enable_effects2.x
#define SCANLINES enable_effects2.y
#define CRT enable_effects2.z
#define TIME enable_effects2.w

#define BLOOM_RADIUS effects1.x
#define BLOOM_INTENSITY effects1.y
#define BLOOM_THRESHOLD effects1.z
#define CHROMATIC_STRENGTH effects1.w

#define VIG_STRENGTH effects2.x
#define VIG_INTENSITY effects2.y
#define NOISE_STRENGTH effects2.z
#define DIST_STRENGTH effects2.w

#define SCAN_STRENGTH effects3.x
#define CRT_CURVE effects3.y
#define NEGATIVE effects3.w

#define ADD_COLOR effects4.x
#define ADD_R effects4.y
#define ADD_G effects4.z
#define ADD_B effects4.w

#define SUB_COLOR effects5.x
#define SUB_R effects5.y
#define SUB_G effects5.z
#define SUB_B effects5.w

#define DIFF_COLOR effects6.x
#define DIFF_R effects6.y
#define DIFF_G effects6.z
#define DIFF_B effects6.w

float noise(vec2 uv)
{
	return fract(sin(uv.x*12.9898+uv.y*78.233)*437.585453*TIME);
}

vec4 vhs(vec2 uv)
{
	vec2 tcoord = uv;
	tcoord.x += sin(TIME*noise(uv));
	return texture2D(tex0 , tcoord)*DIST_STRENGTH;
}

float vig(vec2 uv)
{
	uv *= 1.0 - uv;
	return ( pow(uv.x*uv.y*VIG_INTENSITY*10., VIG_STRENGTH) );
}

vec3 chromatic(vec2 uv, float offset)
{
	float _r = texture2D( tex0, vec2(uv.x - offset, uv.y/* + offset / 5*/)).r;
	float _g = texture2D( tex0, uv).g;
	float _b = texture2D( tex0, vec2(uv.x + offset, uv.y/* - offset / 5*/)).b;
	return vec3(_r,_g,_b);
}

/*
vec3 negative(vec2 uv)
{
	float _r = 1 - texture2D(tex0, vec2(uv));
	float _g = 1 - texture2D(tex0, vec2(uv));
	float _b = 1 - texture2D(tex0, vec2(uv));
	return vec3(_r,_g,_b);
}
*/

vec2 warp(vec2 uv){
	vec2 delta = uv - 0.5;
	float delta2 = dot(delta.xy, delta.xy);
	float delta4 = delta2 * delta2;
	float delta_offset = delta4 * CRT_CURVE;

	return uv + delta * delta_offset;
}

vec4 blur(vec2 uv, vec2 texel)
{
	float total = 0.;
	float rad = 1.;
	mat2 ang = mat2(.73736882209777832,-.67549037933349609,.67549037933349609,.73736882209777832);
	vec2 point = normalize(fract(cos(uv * mat2(195, 174, 286, 183)) * 742.) - .5) * (BLOOM_RADIUS / sqrt(SAMPLES));
	vec4 amount = vec4(0);

	for (float i = 0.; i < SAMPLES; i++)
	{
		point *= ang;
		rad += 1. / rad;
		vec4 samp = texture2D(tex0, uv + point * (rad - 1.) * texel);

		float lum = (samp.r + samp.g + samp.b) / 3.;
		if (lum > BLOOM_THRESHOLD){ amount += samp * (1. / rad); }
		
		total += (1. / rad);
	}
	amount /= total;
	return amount * BLOOM_INTENSITY;
	/*
	float ResS = resolution.x;
	float ResT = resolution.y;

	vec2 stp0 = vec2(1./ResS, 0.);
	vec2 st0p = vec2(0., 1./ResT);
	vec2 stpp = vec2(1./ResS, 1./ResT);
	vec2 stpm = vec2(1./ResS, -1./ResT);

	vec3 i00 = texture2D(DIFFUSE_TEXTURE, uv).rgb;
	vec3 im1m1 = texture2D(DIFFUSE_TEXTURE, uv-stpp*delta.x).rgb;
	vec3 ip1p1 = texture2D(DIFFUSE_TEXTURE, uv+stpp*delta.x).rgb;
	vec3 im1p1 = texture2D(DIFFUSE_TEXTURE, uv-stpm*delta.x).rgb;
	vec3 ip1m1 = texture2D(DIFFUSE_TEXTURE, uv+stpm*delta.x).rgb;
	vec3 im10 = texture2D(DIFFUSE_TEXTURE, uv-stp0*delta.x).rgb;
	vec3 ip10 = texture2D(DIFFUSE_TEXTURE, uv+stp0*delta.x).rgb;
	vec3 i0m1 = texture2D(DIFFUSE_TEXTURE, uv-st0p*delta.x).rgb;
	vec3 i0p1 = texture2D(DIFFUSE_TEXTURE, uv+st0p*delta.x).rgb;

	vec3 target = vec3(0., 0., 0.);
	target += 1.*(im1m1+ip1m1+ip1p1+im1p1); 
	target += 2.*(im10+ip10+i0p1);
	target += 4.*(i00);
	target /= 16.;
	gl_FragColor = vec4(target, 1.);
	*/
}

vec4 blurm(vec2 uv, vec2 texel)
{
	float total = 0.;
	float rad = 1.;
	mat2 ang = mat2(.73736882209777832,-.67549037933349609,.67549037933349609,.73736882209777832);
	vec2 point = normalize(fract(cos(uv*mat2(195,174,286,183))*742.)-.5)*(BLOOM_RADIUS/sqrt(SAMPLES));
	vec4 amount = vec4(0);

	for ( float i=0.; i<SAMPLES; i++ )
	{
		point*=ang;
		rad+=1./rad;
		vec4 samp = texture2D(tex0, uv + point * (rad-1.) * texel);

		float mul = 1.;
		float lum = ( samp.r+samp.g+samp.b )/3.;
		if ( lum < BLOOM_THRESHOLD ){ mul = 0.; }

		amount += samp*(1./rad)*mul;
		total+=(1./rad);
	}
	amount /= total;
	return amount*BLOOM_INTENSITY;
}

void main()
{
	vec2 uv = var_texcoord0;
	if (CRT > 0.5) uv = warp(uv);
	vec4 tint_pm = vec4(tint.xyz * tint.w, tint.w);
	vec4 color = texture2D(tex0, uv) * tint_pm;

	if (CHROMATIC > 0.5) color.rgb = tint.rgb * chromatic(uv, CHROMATIC_STRENGTH * 0.01);
	if (SCANLINES > 0.5) color.rgb *= (1.-SCAN_STRENGTH)+(sin(uv.y*1024.)*SCAN_STRENGTH);
	if (BLOOM > 0.5) color.rgb += blur(uv, uv).rgb;
	/*
	if (BLOOM > 0.5)
	{
		
	}
	*/
	if (NOISE > 0.5) color.rgb += noise(uv) * NOISE_STRENGTH;
	if (VHS > 0.5) color += vhs(uv);
	if (VIGNETTE > 0.5) color.rgb *= vig(uv);

	// cutoff edges
	if (CRT > 0.5 && ( uv.x<0.||uv.y<0.||uv.x>1.||uv.y>1. ))
	{
		color.rgb *= 0.;
	}

	if (ADD_COLOR > 0.5)
	{
		color.rgb = vec3(color.r + ADD_R, color.g + ADD_G, color.b + ADD_B);
	}

	if (SUB_COLOR > 0.5)
	{
		color.rgb = vec3(color.r - SUB_R, color.g - SUB_G, color.b - SUB_B);
	}

	if (DIFF_COLOR > 0.5)
	{
		color.rgb = vec3(DIFF_R - color.r, DIFF_G - color.g, DIFF_B - color.b);
	}

	if (NEGATIVE > 0.5) color.rgb = 1 - color.rgb;

	// Pre-multiply alpha since all runtime textures already are
	gl_FragColor = vec4(color.rgb,1.0);
}
